package com.iflytek.cloud;


import android.os.Bundle;


/**
 * <h3>通用回调接口</h3>
 * <p>
 * 		通用回调接口用于数据上传等，非特定业务的会话回调，如个性化数据上传等。
 * 同一个函数，在不同的业务中回调的数据将可能代表不一样的意义。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see DataUploader#uploadData(SpeechListener, String, byte[])
 * @see SpeakerVerifier#sendRequest(String, String, SpeechListener)
 * @see SpeakerVerifier#getPasswordList(SpeechListener)
 */
public abstract interface SpeechListener {

	/**
	 * <h4>事件回调</h4>
	 * <p>
	 * 		事件回调通过应用层相关的事件消息，根据不同的业务，回调的事件类型
	 * 可能不一样。
	 * </p>
	 * 
	 * @param eventType 消息类型
	 * @param params 消息数据对象
	 * 
	 * @see SpeechEvent
	 */
	void onEvent(int eventType,Bundle params);
	
	/**
	 * <h4>数据回调</h4>
	 * <p>
	 * 		数据回调，根据业务类型不同，可能回调多次，也可能一次也不回调。 
	 * </p>
	 * 
	 * @param buffer 服务端返回的数据，如在声纹获取密码时，表示密码数据。
	 */ 
	void onBufferReceived(byte[] buffer);
	
	/**
	 * <h4>结束回调</h4>
	 * <p>
	 * 		 回调本函数时，当前监听的会话结束。当本函数的参数1非null时，表示当前
	 * 会话有错误，具体错误参考{@link SpeechError}；当参数1为null时，表示当前
	 * 会话成功。
	 * </p>
	 *
	 * @param error 错误信息
	 * @see SpeechError
	 */
	void onCompleted(SpeechError error);
}
