package com.iflytek.cloud.msc.isv;


import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.concurrent.ConcurrentLinkedQueue;

import android.content.Context;
import android.media.MediaRecorder;
import android.media.MediaRecorder.AudioSource;
import android.os.Bundle;
import android.os.HandlerThread;
import android.os.Message;
import android.os.SystemClock;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechEvent;
import com.iflytek.cloud.VerifierListener;
import com.iflytek.cloud.VerifierResult;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.MscSession.ResultStatus;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.NetworkUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.record.FilePcmRecorder;
import com.iflytek.cloud.record.PcmRecorder;
import com.iflytek.cloud.record.PcmRecorder.PcmRecordListener;

/**
 * Resognize thread,manager record and msc interface
 * 
 * @author iflytek
 * 
 */

public class MscVerify extends MscHandler implements PcmRecordListener {

	public static final String KEY_SST = "sst=";
	public static final String SST_TRAIN = "train";
	public static final String SST_VERIFY = "verify";
	public static final String SST_IDENTITY = "identify";

	protected volatile 	 VerifierListener mListener = null;

	protected long       mRecordBegin   = 0;

	//是否是听写会话，听写和识别设置的vad超时时间不同
	protected boolean      mLongTimeInput     = true;

	protected VerifySession  mSession = new VerifySession();
	protected PcmRecorder mRecorder = null;

	// 业务类型,操作的类型有train（声纹密码注册）、login（声纹密码登陆）、verify（声纹密码确认）和identity（声纹密码鉴别）。
	protected String mSst = SST_TRAIN;
	// 用户声纹id列表
	protected String mVidlist = "";
	// 声纹结果
	protected VerifierResult  mResult = null;

	/*录音缓存队列，用于录音回放和文件保存*/
	protected ConcurrentLinkedQueue<byte[]> mRecordBuffer = null;
	// 录音源
	protected int  mAudioSource = AudioSource.MIC;
	private long mTime = 0;
	// 需要过滤掉的音频块大小
	private int filter_audio = 0;
	
	private String mSessionID = null;
	
	private boolean mUseBluetooth = false;

	public MscVerify(Context context,HashParam param,HandlerThread mHandlerThread) {
		super( context, mHandlerThread);
		mRecordBuffer = new ConcurrentLinkedQueue<byte[]>();
		setParams(param);
	}

	public synchronized void startListening(VerifierListener listener) {
		DebugLog.LogD("Isv Msc startVerify in");
		mListener = listener;
		// 启动会话
		start();
		DebugLog.LogD("Isv Msc startVerify out");
	}

	public synchronized boolean stopListening() {
		DebugLog.LogD("Isv Msc stopRecord in");
		if (getStatus() != Status.recording) {
			DebugLog.LogD("endVerify fail  status is :" + getStatus());
			return false;
		}
		if(!isRegister())
			releaseRecord();

		//TODO 赵艳军修改，未完成
		sendMsg(MSG_RECORD_STOPD);

		DebugLog.LogD("Isv Msc stopRecord out");
		return true;

	}

	/**
	 * 是否注册流程,注册时一直启动录音，不频繁启动
	 * @return
	 */
	private boolean isRegister()
	{
		String sst = getParam().getString(SpeechConstant.ISV_SST);
		return SST_TRAIN.equalsIgnoreCase(sst);
	}

	@Override
	public void cancel(boolean notifyError) {
		if(notifyError && isRunning() && mListener != null){
			mListener.onError(new SpeechError(ErrorCode.ERROR_INTERRUPT));
		}
		
		releaseRecord();
		
		super.cancel(notifyError);
	}




	@Override
	protected void onMsgProcess(Message msg) throws Throwable, SpeechError {

		super.onMsgProcess(msg);

		switch( msg.what )
		{
		case MSG_START:
			proc_Msg_Start();
			break;
		case MSG_SESSION_BEGIN:
			proc_Msg_Session_Begin();
			break;
		case MSG_RECORD_AUDIO:
			proc_Msg_Record_Data(msg);
			break;
		case MSG_RECORD_STOPD:
			proc_Msg_Record_Stoped();
			break;
		case MSG_RESULT:
			proc_Msg_Result(msg);
			break;
		case MSG_SPEECH_TIMEOUT:
			vadEndCall();
		case MSG_NET_PERF:
			//proc_Msg_Netperf();
			break;
		default:
			break;
		}
	}


	protected void proc_Msg_Start() throws Exception {
		DebugLog.LogD("isv msc msg start in");
		String engine = getParam().getString(SpeechConstant.ENGINE_TYPE);
		boolean net_check = getParam().getBoolean(SpeechConstant.NET_CHECK, true);
		if(SpeechConstant.TYPE_CLOUD.equals(engine) && net_check)
			NetworkUtil.checkNetwork(mContext);
		
		int read_rate = getParam().getInt(MscKeys.KEY_RECORD_READ_RATE, PcmRecorder.READ_INTERVAL40MS);

		// 判断不是外部写录音会话,声纹只需要开启一次录音机
		if(mAudioSource != MscKeys.VALUE_AUDIO_EXTERNAL && isRunning()){
			DebugLog.LogD( "[isv]start  record");
			if (mRecorder == null ) {
				mUseBluetooth = getParam().getBoolean( SpeechConstant.BLUETOOTH, mUseBluetooth );
				if( mUseBluetooth ){
					startBluetooth();
				}
				
				mRecorder = new PcmRecorder(getSampleRate(), read_rate,mAudioSource);
				mRecorder.startRecording(this);
			}
		}

		if (getStatus() != Status.exiting && null != mListener)
			mListener.onBeginOfSpeech();


		mRecordBegin = SystemClock.elapsedRealtime();

		// 语音输入超时消息
		removeMessages(MSG_SPEECH_TIMEOUT);
		sendMsg(MSG_SPEECH_TIMEOUT, Priority.normal, false,mSpeechTimeOut);
		// 插入到消息头部，防止音频被预先处理
		sendMsg(MSG_SESSION_BEGIN,Priority.max,false,0);
		
		DebugLog.LogD("isv msc msg start out");
	}

	/**
	 * MSC SessionBegin消息处理
	 * 特别注意点：缓冲音频的写入，必须保证在SessionBegin之后
	 * @throws Exception
	 */
	protected void proc_Msg_Session_Begin() throws Exception {
		// 注册时不需要多次启动会话，后续需要考虑sessionBegin失败情况！！
		if(mSession.mClientID == null){
			//记录会话开始信息
			PerfLogger.appendInfo(PerfLogger.SDK_SESSION_BIGNE, null);
			mSession.sessionBegin(mContext, mVidlist, this);
		}
		setStatus(Status.recording);
	}
	/**
	 * 录音数据通知消息
	 * @param msg 消息实体
	 * @throws Exception
	 */
	protected void proc_Msg_Record_Data(Message msg) throws Exception{

		byte[] data = (byte[])msg.obj;
		if (data == null || data.length == 0)
			return;

		// 保存录音数据，用以再次识别和保存
		mRecordBuffer.add(data);

		uploadData(data, true);
	}



	/**
	 * 录音停止的通知消息
	 * @throws SpeechError
	 * @throws IOException
	 * @throws InterruptedException
	 */
	private void proc_Msg_Record_Stoped() throws SpeechError, IOException, InterruptedException{	
		DebugLog.LogD("record stop msg in");
		if(!isRegister())
			releaseRecord();

		// upload end flag
		mSession.pushEndFlag();

		// 最低优先级消息，会被自动清除
		//sendMsg(MSG_NET_TIMEOUT, Priority.min, false, mTimeOut);
		sendMsg(MSG_RESULT);

		DebugLog.LogD("record stop msg out");
	}


	/**
	 * 结果通知消息
	 * 	status:   2----unknown 0----has result 5----is last result
	 * @throws SpeechError
	 * @throws InterruptedException
	 * @throws UnsupportedEncodingException
	 */
	void proc_Msg_Result(Message msg) throws SpeechError, InterruptedException, UnsupportedEncodingException{
		if(!isRegister())
			releaseRecord();
		requestResult();

		if (getStatus() == Status.waitresult){
			sendMsg(MSG_RESULT, Priority.normal, false, 20);
		}
		//		DebugLog.Log(TAG, "mStatusBegin:" + mStatusBegin 
		//						 + "Current:" + SystemClock.elapsedRealtime() 
		//						 + " TimeOut=" + mTimeOut);
		//timeOutCheck(mStatusBegin,mTimeOut);
	}	

	@Override
	protected void onEnd(SpeechError error) {
		DebugLog.LogD("isv msc onEnd in");
		releaseRecord();
		// has to end isv session,or will occur error
		getSessionID();

		PerfLogger.appendInfo(PerfLogger.SESSION_END_BEGIN, null);

		if(mUserCancel)
			mSession.sessionEnd("user abort");
		else if(error != null)
			mSession.sessionEnd("error" + error.getErrorCode());
		else
			mSession.sessionEnd("success");
		PerfLogger.appendInfo(PerfLogger.SESSION_END_END, null);

		super.onEnd(error);

		if(mListener != null)
		{
			if(mUserCancel)	// cancel by user
			{
				//DebugLog.LogD("VerifyListener#onCancel");
				//				mListener.onCancel();
			}else	// auto exit
			{
				DebugLog.LogD("VerifyListener#onEnd");
				
				if(error != null) {
					// 通过onEvent抛出服务端会话id
					Bundle bundle = new Bundle();
					bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, getSessionID());
					mListener.onEvent(SpeechEvent.EVENT_SESSION_ID, 0, 0, bundle);
					
					mListener.onError(error);
				}
			}
		}
		mListener = null;
		DebugLog.LogD("isv msc onEnd out");
	}

	/**
	 * @throws SpeechError 
	 * @throws UnsupportedEncodingException 
	 * 
	 */
	private void requestResult() throws SpeechError, UnsupportedEncodingException{
		ResultStatus vstatus = mSession.getStatus();

		switch (vstatus) {
		case noResult:
			break;
		case hasResult:
			notifyResult();
			break;
		default:
			break;
		}
	}

	/**
	 * called from VAD check
	 */
	public void vadEndCall() {
		if (Status.recording == getStatus()) {
			DebugLog.LogD("Isv Msc vadEndCall");
			stopListening();
			if (mListener != null)
				mListener.onEndOfSpeech();
		}
	}

	private void notifyResult() throws SpeechError, UnsupportedEncodingException{
		mStatusBegin = SystemClock.elapsedRealtime();
		String text = new String(mSession.getResultData(),DataUtil.UTF8);
		mResult = new VerifierResult(text);
		
		if (null != mListener) {
			// 通过onEvent抛出服务端会话id
			Bundle bundle = new Bundle();
			bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, getSessionID());
			mListener.onEvent(SpeechEvent.EVENT_SESSION_ID, 0, 0, bundle);
		}
		
		
		if(mSst.equals(SST_TRAIN))
		{
			// 注册次数尚未完成
			if(mResult.ret == 0 && mResult.suc < mResult.rgn)
			{
				if(mListener != null) {
					PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
					mListener.onResult(mResult);
				}
				sendMsg(MSG_START);
				return;
			}
		}
		if(mListener != null) {
			PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
			mListener.onResult(mResult);
		}

		
		exit(null);
	}


	@Override
	protected void onParseParam() {
		mSpeechTimeOut = getParam().getInt(SpeechConstant.KEY_SPEECH_TIMEOUT, mSpeechTimeOut);
		mVidlist = getParam().getString(SpeechConstant.ISV_VID);
		mAudioSource =  getParam().getInt(SpeechConstant.AUDIO_SOURCE, MediaRecorder.AudioSource.MIC);
		int filterTime = getParam().getInt(SpeechConstant.FILTER_AUDIO_TIME, 0);
		filter_audio = getParam().getInt(SpeechConstant.SAMPLE_RATE, mSampleRate)/1000*16/8*filterTime;
		DebugLog.LogD("mSpeechTimeOut=" + mSpeechTimeOut);
		super.onParseParam();
	}


	private void releaseRecord()
	{
		// stop audio record
		if (mRecorder != null){
			mRecorder.stopRecord(getParam().getBoolean(MscKeys.RECORD_FORCE_STOP, false));
			mRecorder = null;
			
			if( mUseBluetooth ){
				stopBluetooth();
			}
		}
	}
	
	void onWaiting() throws SpeechError, InterruptedException, UnsupportedEncodingException{
		if(!isRegister())
			releaseRecord();
		requestResult();
		if (getStatus() == Status.waitresult){
			Thread.sleep(20);
		}

		timeOutCheck(mStatusBegin,mNetTimeOut);
	}
	/**
	 * 通知音量变化
	 * @param data
	 * @param volume
	 */
	public void callbackVolume(byte[] data, int volume) {
		if (isRunning())
		{
			mListener.onVolumeChanged(volume,data);
		}
	}

	protected void uploadData(byte[] data,boolean needvadNotify) 
			throws SpeechError{	
		//upload record data,and get result if has
		//long	d1 = System.currentTimeMillis();		
		mSession.pushAudioData(data, data.length);

		// check vad state
		if (needvadNotify) {
			if (mSession.getVadEnd()) {
				vadEndCall();
			} else {
				int volume = mSession.getAudioVolume();
				callbackVolume(data, volume);
			}
		}
	}
	/**
	 * 录音缓存队列
	 * @return
	 */
	public ConcurrentLinkedQueue<byte[]> getBuffer() {
		return mRecordBuffer;
	}
	/**
	 * record callback from PcmRecorder
	 * 
	 * @throws IOException
	 */
	@Override
	public void onRecordBuffer(byte[] data,int offset,int length) {
		if (Status.recording != getStatus()) 
			return;

		if (length > 0) {
			if (filter_audio > 0) { //需要过滤掉的音频块大小
				if (filter_audio >= length) {
					filter_audio -= length;
				}else {
					// 拷贝内存，发送录音消息
					byte[] temp = new byte[length-filter_audio];
					System.arraycopy(data, offset+filter_audio, temp, 0, length-filter_audio);
					sendMsg(obtainMessage(MSG_RECORD_AUDIO,temp));
					filter_audio = 0;
				}
			}else {
				// 拷贝内存，发送录音消息
				byte[] temp = new byte[length];
				System.arraycopy(data, offset, temp, 0, length);
				sendMsg(obtainMessage(MSG_RECORD_AUDIO,temp));
			}
		}
	}

	@Override
	public void onError(SpeechError e) {
		exit(e);
	}

	@Override
	public void onRecordStarted(boolean success) {
		// TODO Auto-generated method stub
		DebugLog.LogD("time cost: onRecordStarted:" + (SystemClock.elapsedRealtime() - mTime));
	}
	/**
	 * 获取录音源
	 * @return 示例：-1、MIC
	 */
	public int getAudioSource()
	{
		return mAudioSource;
	}
	@Override
	public void onRecordReleased() {
		if(mRecorder != null && mRecorder instanceof FilePcmRecorder)
		{
			//文件录音释放录音机则有可能是文件读完，模拟手动停止录音
			stopListening();
		}
	}
	/**
	 * 返回客户端ID
	 */
	public String getClientID() {
		return mSession.getClientID();
	}
	/**
	 * 返回服务端ID
	 */
	@Override
	public String getSessionID()
	{
		if( TextUtils.isEmpty(mSessionID) ){
			mSessionID = mSession.getSessionID();
		}
		return mSessionID;
	}

}