package com.iflytek.cloud.msc.ivw;


import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.json.JSONObject;
import android.content.Context;
import android.media.MediaRecorder;
import android.media.MediaRecorder.AudioSource;
import android.os.Bundle;
import android.os.HandlerThread;
import android.os.Message;
import android.text.TextUtils;
import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.RecognizerResult;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechEvent;
import com.iflytek.cloud.WakeuperListener;
import com.iflytek.cloud.WakeuperResult;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.util.Config;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.FileUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.record.PcmRecorder;
import com.iflytek.cloud.record.PcmRecorder.PcmRecordListener;
import com.iflytek.msc.MSC;

/**
 * Ivw thread,manager record and msc interface
 * 
 * @author iflytek
 * 
 */

public class MscWakeuper extends MscHandler implements PcmRecordListener{
	// 业务类型,操作的类型有enroll(训练)和wake（唤醒）oneshot(唤醒+识别)
	public String session_type ;

	public static enum IvwMessage{
		undefined_0
		, IVW_MSG_WAKEUP
		, IVW_MSG_ERROR
		, IVW_MSG_ISR_RESULT 
		, IVW_MSG_ISR_EPS 
		, IVW_MSG_VOLUME
		, IVW_MSG_ENROLL
		, IVW_MSG_RESET 
	}
	
	// 结果类型：唤醒结果 0,识别结果 1
	public static final int IVW_RESULT = 0;
	public static final int REC_RESULT = 1;
	public static final int ENROLL_RESULT = 2;

	// 唤醒状态
	public static final String IVW_STATUS = "status";
	public static final String IVW_STATUS_DONE = "done";
	
	//唤醒音频
	public static final String KEY_WAKEUP_AUDIO = "ivw_audio";
	
	// 外部回调接口
	protected volatile 	 WakeuperListener mListener = null;

	// 是否用户主动点击了停止录音
	protected boolean    mUstopRecord = false;
	// 录音源
	protected int	mAudioSource = AudioSource.MIC;
	
	protected IvwSession  mSession = new IvwSession();
	protected PcmRecorder mRecorder = null;
	/*识别结果列表,保存的为原始结果*/
	protected ArrayList<String> mResults = null;
	
	// 是否已唤醒（唤醒+识别时需要用来做状态判断）
	public boolean isWake = false;
	// 是否保持持续唤醒状态
	public boolean keep_alive = false;
	
	// 录音缓存队列，用于录音回放和文件保存
	protected final ConcurrentLinkedQueue<byte[]> mAudioSavedBuffer = new ConcurrentLinkedQueue<byte[]>();
	protected final int AUDIO_SAVED_BUF_TIME = 1000*60;
	protected String mAudioSavedPath = null;
	protected int mAudioSavedMaxByte = 0;
	protected int mCurrentAudioSavedByte = 0;
	
	protected boolean mRecordDataNotified = false;
	
	protected final int AUW_MAX_BUF_TIME = 2000;
	protected int mAuwMaxBufSize = 0;
	protected long mCurSyncAuwSize = 0;
	protected long mCurRecSize = 0;
	protected long mCurWritenSize = 0;
	
	protected final int MIN_LOG_PERIOD = 100;
	protected int mLogCount = 0;
	
	private boolean mUseBluetooth = false;
			
 	public MscWakeuper(Context context,HashParam param,HandlerThread mHandlerThread) {
		super( context, mHandlerThread);
		mUstopRecord = false;
		setParams(param);
		mResults = new ArrayList<String>();
	}
	
	/**
	 * 获取录音源
	 * @return 示例：-1、MIC
	 */
	public int getAudioSource()
	{
		return mAudioSource;
	}

	/**
	 * 开始识别
	 * @param listener 唤醒监听器
	 */
	public synchronized void startListening(WakeuperListener listener) {
		// 启动会话消息
        mListener = listener; 
		DebugLog.LogD("[ivw]startListening called");
		// 启动会话
		start();
	}
	
	@Override
	protected void onMsgProcess(Message msg) throws Throwable, SpeechError {
		super.onMsgProcess(msg);
		
		switch( msg.what )
		{
			case MSG_START:
				proc_Msg_Start();
				break;
			case MSG_SESSION_BEGIN:
				proc_Msg_Session_Begin();
				break;
			case MSG_RECORD_AUDIO:
				proc_Msg_Record_Data(msg);
				break;
			case MSG_RECORD_STOPD:
				proc_Msg_Record_Stoped();
				break;
			case MSG_RESULT:
				proc_Msg_Result(msg);
				break;
			case MSG_NET_PERF:
				break;
			default:
				break;
		}
	}

	/**
	 * 启动消息处理
	 * @throws Exception
	 */
	protected void proc_Msg_Start() throws Exception {	
		DebugLog.LogD("[ivw]start connecting");
		//初始化唤醒状态
		isWake = false;
		int read_rate = getParam().getInt(MscKeys.KEY_RECORD_READ_RATE, PcmRecorder.READ_INTERVAL40MS);
		mRecordDataNotified = getParam().getBoolean( SpeechConstant.NOTIFY_RECORD_DATA, mRecordDataNotified );

		// 判断不是外部写录音会话
		if(mAudioSource != MscKeys.VALUE_AUDIO_EXTERNAL && isRunning()){
			this.mAudioSavedPath = getParam().getString( SpeechConstant.IVW_AUDIO_PATH );
			if( !TextUtils.isEmpty(mAudioSavedPath) ){
				this.mAudioSavedMaxByte = AUDIO_SAVED_BUF_TIME * getSampleRate()/1000 * 2; 
			}
			
			DebugLog.LogD( "[ivw]start  record");
			if (mRecorder == null ){ 
				mUseBluetooth = getParam().getBoolean( SpeechConstant.BLUETOOTH, mUseBluetooth );
				if( mUseBluetooth ){
					startBluetooth();
				}
				
				mRecorder = new PcmRecorder(getSampleRate(), read_rate,mAudioSource);
				mRecorder.startRecording(this);
			}
		}else{
			mAudioSavedMaxByte = 0;
		}
		
		mAuwMaxBufSize = AUW_MAX_BUF_TIME * getSampleRate()/1000 * 2;
		DebugLog.LogD( "max saved buf byte: "+this.mAudioSavedMaxByte + ", max auw buf byte: "+this.mAuwMaxBufSize );
		
		if (getStatus() != Status.exiting && null != mListener)
			mListener.onBeginOfSpeech();
		// 插入到消息头部，防止音频被预先处理
		sendMsg(MSG_SESSION_BEGIN,Priority.max,false,0);
	}
	
	private int mSessionRetryCount = 0;
	/**
	 * MSC SessionBegin消息处理
	 * 特别注意点：缓冲音频的写入，必须保证在SessionBegin之后
	 * @throws Exception
	 */
	protected void proc_Msg_Session_Begin() throws SpeechError, Throwable {
		int code = ErrorCode.SUCCESS;
		// 训练时不需要多次启动会话
		if(mSession.mClientID == null){
			PerfLogger.appendInfo(PerfLogger.SDK_SESSION_BIGNE, null);
			code = mSession.sessionBegin(mContext,session_type, this);
			if(code == 0 && null != mSession.mClientID){
				//注册回调接口
				if(isRunning()){
					MSC.QIVWRegisterNotify(mSession.mClientID, "MsgProcCallBack", this);
					setStatus(Status.recording);
				}
			}else if(ErrorCode.SUCCESS!=code) {
				// 延迟消息无法发送,必须通过sleep方式，否则会造成音频丢失或处理错误
				// 需要优先级放到最高，否则会造成录音消息被预先处理，造成报错
				mSessionRetryCount++;
				if(mSessionRetryCount > 40){
					throw new SpeechError(code);
				}else if(isRunning())	
				{
					Thread.sleep(15);
					sendMsg(MSG_SESSION_BEGIN, Priority.max,false,0);
				}
			}else{
				DebugLog.LogE( "current csid: "+mSession.mSessionID );
				throw new SpeechError( ErrorCode.ERROR_UNKNOWN );
			}
		}
	}
	
	/**
	 * 录音数据通知消息
	 * @param msg
	 * @throws Exception
	 */
	protected void proc_Msg_Record_Data(Message msg) throws Throwable{

		byte[] data = (byte[])msg.obj;
		if (data == null || data.length == 0){
			return;
		}
		
		uploadData(data, true);
		mCurWritenSize += data.length;
		if( this.mCurSyncAuwSize < this.mCurRecSize ){
			this.mCurSyncAuwSize += data.length;
		}
		
		++mLogCount;
		
		// 判断是否需要清空缓存
		if( this.mCurRecSize-this.mCurSyncAuwSize >= this.mAuwMaxBufSize ){
			DebugLog.LogW( "cur rec buf: "+mCurRecSize+", cur sync auw size: "+mCurSyncAuwSize+", cur writen size: "+mCurWritenSize
					+", diff match max buf size: "+mAuwMaxBufSize+", cur bufs in msg will be ignored!" );
			this.mCurSyncAuwSize = this.mCurRecSize;
			removeMessages( MSG_RECORD_AUDIO );
			System.gc();
		}else if( MIN_LOG_PERIOD <= mLogCount ){
			mLogCount = 0;
			DebugLog.LogD( "cur rec buf: "+mCurRecSize+", cur sync auw size: "+mCurSyncAuwSize+", cur writen size: "+mCurWritenSize );
		}
		
		if( 0 < this.mAudioSavedMaxByte ){
			while( mAudioSavedMaxByte < mCurrentAudioSavedByte ){
				final byte[] audioRemoved = mAudioSavedBuffer.poll();
				mCurrentAudioSavedByte -= null!=audioRemoved ? audioRemoved.length : 0;
			}
			
			this.mAudioSavedBuffer.add( data );
			mCurrentAudioSavedByte += data.length;
		}
		
		if( mRecordDataNotified ){
			// 抛出音量、音频
			Bundle bundle = new Bundle();
			bundle.putByteArray(SpeechEvent.KEY_EVENT_RECORD_DATA,data);
			mListener.onEvent(SpeechEvent.EVENT_RECORD_DATA, 0, 0,bundle);
		}		
	}
	/**
	 * msc唤醒结果回调接口
	 * 
	 * @param sessionID 会话id
	 * @param msgCB  1：结果正常 ;2:发生错误
	 * @param param1 错误码
	 * @param param2
	 * @param result 唤醒结果
	 */
	 int MsgProcCallBack(final char[] sessionID, int msgCB, int param1, int param2, final byte[] result)
	{	
		 DebugLog.LogD( "cur rec buf: "+mCurRecSize+", cur sync auw size: "+mCurSyncAuwSize+", cur writen size: "+mCurWritenSize );
		 
		// 唤醒结果回调消息，优先级最高
		if(result != null)
			DebugLog.LogD("MscWakeuper", "msg:"+msgCB + "param1:"+param1 + "param2:"+param2 + "result:" + new String( result ));
		else
			DebugLog.LogD("MscWakeuper", "msg:"+msgCB + "param1:"+param1 + "param2:"+param2 + "result:null");
		
		IvwMessage ivwMsg = null;
		try{
			ivwMsg = IvwMessage.values()[ msgCB ];
		}catch( Throwable e ){
			DebugLog.LogE( "unmatched ivw message!" );
			DebugLog.LogE( e );
		}//end of try-catch
		
		if( null != ivwMsg ){
			switch (ivwMsg) {
			// 唤醒结果
			case IVW_MSG_WAKEUP:
				isWake = true;
				Message msg = null;
				if(keep_alive || session_type.equals(MscKeys.IVW_ONESHOT))
				{
					// 设置为一直唤醒、唤醒+识别时，不终止会话
					msg = obtainMessage(MSG_RESULT, 0, IVW_RESULT, result);
				}
				else 
				{
					msg = obtainMessage(MSG_RESULT, 5, IVW_RESULT, result);
				}
				//消息队列在执行很耗时的任务 后至的结果数据被插入队列前，导致之前的结果无法展示
				//ps:概率很小
				if(hasMessages( MSG_RESULT ))
					sendMsg(msg, Priority.normal, false, 0);
				else
					sendMsg(msg, Priority.max, false, 0);
				break;
			// 发生错误
			case IVW_MSG_ERROR:
				onError(new SpeechError(param1));	
				break;
			// 识别结果
			case IVW_MSG_ISR_RESULT:
				Message msgRec = null;
				msgRec = obtainMessage(MSG_RESULT, param1, REC_RESULT, result);
				//消息队列在执行很耗时的任务 后至的结果数据被插入队列前，导致之前的结果无法展示
				//ps:概率很小
				if(hasMessages( MSG_RESULT ))
					sendMsg(msgRec, Priority.normal, false, 0);
				else
					sendMsg(msgRec, Priority.max, false, 0);
				break;
			// 端点信息
			case IVW_MSG_ISR_EPS:
				// 检测到后端点
				if (param1 == 3) {
					vadEndCall();
				}
				break;
			
			//注册的音量回调
			case IVW_MSG_VOLUME:
				try{
					if(mListener != null)
					{
						mListener.onVolumeChanged(param1);
					}
				}catch (Exception e) {
				}
				
				break;
			// 注册结果
			case IVW_MSG_ENROLL:
				Message enrollMsg = obtainMessage(MSG_RESULT, 0, ENROLL_RESULT, result);
				sendMsg(enrollMsg, Priority.max, false, 0);
				break;
			default:
				break;
			}
		}//end of ivwMsg not null
		
		return 0;
	}

	 /**
	 * 停止录音
	 * @return
	 */
	public synchronized boolean stopListening(boolean userStop) {
		DebugLog.LogD("stopListening, current status is :" + getStatus()+" usercancel : " +userStop);
		//注册不需要关闭录音，三遍成功后内部关闭
		if(session_type.equals(MscKeys.IVW_ENROLL)){
			mUstopRecord = userStop;
			sendMsg(MSG_RECORD_STOPD);
		}
		//业务类型为唤醒+听写/识别  且 已经被唤醒  
		else if (session_type.equals(MscKeys.IVW_ONESHOT) && isWake) {
			releaseRecord();
			// 用户主动点击录音停止
			mUstopRecord = userStop;
			sendMsg(MSG_RECORD_STOPD);
		}else {//唤醒时调用stopListening直接取消
			cancel(false);
		}
		return true;
	}
	
	/**
	 * 录音停止的通知消息
	 * @throws SpeechError
	 * @throws IOException
	 * @throws InterruptedException
	 */
	private void proc_Msg_Record_Stoped() throws SpeechError, IOException, InterruptedException{	
		DebugLog.LogD( "recording stop" );
		if(!session_type.equals(MscKeys.IVW_ENROLL)){
			releaseRecord();
		}
		mSession.pushEndFlag();
	}
	
	/**
	 * 取消唤醒
	 */
	@Override
	public void cancel(boolean notifyError) {
		if(notifyError && isRunning() && mListener != null){
			mListener.onError(new SpeechError(ErrorCode.ERROR_INTERRUPT));
		}
		DebugLog.LogD( "cancel" );
		releaseRecord();
		// 用户主动点击取消
		if(getStatus() == Status.recording){
			mUstopRecord = true;
		}
		super.cancel(notifyError);
	}
	
	@Override
	protected void onEnd(SpeechError error) {
		DebugLog.LogD("onSessionEnd");
		releaseRecord();
		
        // 保存到本地文件中,只有设置文件路径生效
        if( !TextUtils.isEmpty(this.mAudioSavedPath) ){
        	if( FileUtil.saveFile(this.mAudioSavedBuffer, mAudioSavedPath) ){
        		final String format = this.getParam().getString(SpeechConstant.AUDIO_FORMAT, null);
        		final int sample = this.getSampleRate();
        		FileUtil.formatPcm(format, mAudioSavedPath, sample);
        		DebugLog.LogD( "save ivw audio succeed: "+mAudioSavedPath );
        	}else{
        		DebugLog.LogE( "save ivw audio failed: "+mAudioSavedPath );
        	}
        }
		
		// 如果无结果，判断是否需要通知10118错误,默认不返回
		if(session_type.equals(MscKeys.IVW_ONESHOT) && isWake && mResults.size() <= 0 && 
				error == null && getParam().getBoolean(SpeechConstant.ASR_NOMATCH_ERROR, true))
		{
			error = new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
		}
		
		PerfLogger.appendInfo(PerfLogger.SESSION_END_BEGIN, null);
		if(mUserCancel)
			mSession.sessionEnd("user abort");
		else if(error != null)
			mSession.sessionEnd("error" + error.getErrorCode());
		else
			mSession.sessionEnd("success");
		PerfLogger.appendInfo(PerfLogger.SESSION_END_END, null);
		
		super.onEnd(error);
		
		if(mListener != null)
		{
			if(mUserCancel)	// cancel by user
			{
				DebugLog.LogD("WakeuperListener#onCancel");
			}else	// auto exit
			{
				DebugLog.LogD("WakeuperListener#onEnd");
				if(error != null)
					mListener.onError(error);
			}
		}
		mListener = null;
	}

	/**
	 * 通知外部识别结果
	 * @param isLast
	 * @throws SpeechError
	 * @throws UnsupportedEncodingException
	 */
	private void notifyResult(boolean isLast,  byte[] resultByte,int reslutType, Bundle extra) throws SpeechError, UnsupportedEncodingException{
		//识别结果
		if (reslutType == REC_RESULT)
		{
			String result = null;
			if(resultByte != null && resultByte.length > 0)
			{
				result = new String(resultByte, getResultEncoding());
			}
			else if(mResults.size() <= 0)
			{
				// 本地识别无匹配结果时v5+给出空结果,返回 ERROR_NO_MATCH
				String localGrammar  = getParam().getString(SpeechConstant.LOCAL_GRAMMAR);
				if (!TextUtils.isEmpty(localGrammar) && !"sms.irf".equals(localGrammar))
				{
					throw new SpeechError(ErrorCode.ERROR_NO_MATCH);
				}
				else 
				{
					throw new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
				}
			}
			else 
			{
				result = "";
			}
			// 保存识别结果,用以判断是否返回10118错误
			mResults.add(result);
			if(null != mListener && isRunning())
			{
				Bundle bundle = new Bundle();
				bundle.putParcelable(SpeechEvent.KEY_EVENT_IVW_RESULT, new RecognizerResult(result));
				mListener.onEvent(SpeechEvent.EVENT_IVW_RESULT, isLast?1:0 ,0, bundle);
			}
			DebugLog.LogD("msc result time:" + System.currentTimeMillis());
		}
		//唤醒结果
		else if (reslutType == IVW_RESULT)
		{
			String result = null;
			if(resultByte != null && resultByte.length > 0)
			{
				result = new String(resultByte, DataUtil.UTF8);
			}
			else 
			{
				throw new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
			}

			if(null != mListener && isRunning())
			{
				// 原始结果
				byte[] audio = null;
				if( null != extra ){
					audio = extra.getByteArray( KEY_WAKEUP_AUDIO );
				}
				WakeuperResult reresult = new WakeuperResult(result, audio);
				PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
				mListener.onResult(reresult);
			}

		}
		//注册结果
		else if(reslutType == ENROLL_RESULT)
		{
			boolean last = false;
			
			String result = null;
			if(resultByte != null && resultByte.length > 0)
			{
				result = new String(resultByte, DataUtil.UTF8);
			}
			else 
			{
				throw new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
			}

			if(session_type.equals(MscKeys.IVW_ENROLL))
			{
				if(mListener != null)
				{
					// 原始结果
					WakeuperResult enrollResult = new WakeuperResult(result);
					mListener.onResult(enrollResult);
				}
				
				try {
					JSONObject jsonRst 	= new JSONObject(result);
					// 注册次数尚未完成
					if(jsonRst.getInt("suc_times") != jsonRst.getInt("total_times"))
					{
						sendMsg(MSG_START);
					}
					else
					{
						last = true;
					}
				}
				catch(Exception e){
					throw new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
				}
			}
			
			if(null != mListener && isRunning())
			{
				// 原始结果
				WakeuperResult reresult = new WakeuperResult(result);
				mListener.onResult(reresult);
				if (last)
				 exit(null);
			}
		}

		if(isLast)
		{
			exit(null);
		}
	}	
	
	@Override
	protected void onParseParam() {
		//获取业务类型
		session_type = 	getParam().getString(SpeechConstant.IVW_SST,MscKeys.IVW_WAKE);
		//是否保持一直唤醒
		keep_alive = getParam().getBoolean(SpeechConstant.KEEP_ALIVE,false);
		//获取音频源
		mAudioSource =  getParam().getInt(SpeechConstant.AUDIO_SOURCE, MediaRecorder.AudioSource.MIC);
		// 设置唤醒筛选区间
		int netval = Config.getConfig(mContext).getInt(MscKeys.IVW_NETVAL, 20);
		getParam().putParam(MscKeys.IVW_NETVAL, netval+"", false);
		super.onParseParam();
	}
	
	@Override
	public boolean isLongInput() {
		return false;
	}

	/**
	 * stop listen message
	 * @throws SpeechError 
	 * @throws IOException 
	 * @throws InterruptedException 
	 */

	private void releaseRecord()
	{
		if (mRecorder != null){
			mRecorder.stopRecord(getParam().getBoolean(MscKeys.RECORD_FORCE_STOP, false));
			mRecorder = null;
			
			if( mUseBluetooth ){
				stopBluetooth();
			}
		}
	}
	/**
	 * 结果通知消息
	 * 	status: 0----has result 5----is last result
	 * @throws SpeechError
	 * @throws InterruptedException
	 * @throws UnsupportedEncodingException
	 */
	void proc_Msg_Result(Message msg) throws SpeechError, InterruptedException, UnsupportedEncodingException{
		
		int status = msg.arg1;
		byte[] data = (byte[])msg.obj;
		
		switch (status) {
			case 0:
				notifyResult(false, data,msg.arg2, msg.getData() );
				break;
			case 2:
				throw new SpeechError(ErrorCode.ERROR_FILE_ACCESS); 
			case 5:
				notifyResult(true, data,msg.arg2, msg.getData() );
				break;
			default:
				break;
		}
	}
	
	/**
	 * 传入音频
	 * @param data
	 * @param needvadNotify
	 * @throws SpeechError
	 */
	protected void uploadData(byte[] data,boolean needvadNotify) 
		throws SpeechError{
		mSession.pushAudioData(data, data.length);
	}
	
	public int writeAudio( byte[] data,int offset,int length ){
		onRecordBuffer( data, offset, length );
		return ErrorCode.SUCCESS;
	}

	/**
	 * record callback from PcmRecorder
	 * 
	 * @throws IOException
	 */
	@Override
	public void onRecordBuffer(byte[] data,int offset,int length) {
		// record data notify
		if(data.length < length || data == null || length <= 0)
			return;
		if (length > 0 && isRunning()) {
			this.mCurRecSize += length;

			// 拷贝内存，发送录音消息
			byte[] temp = new byte[length];
			System.arraycopy(data, offset, temp, 0, length);
			if(mListener != null) {
				double volume = getVolume(length,temp);
				mListener.onVolumeChanged((int)volume);
			}
			sendMsg(obtainMessage(MSG_RECORD_AUDIO,temp));
		}
	}

	/**
	 * 录音错误
	 */
	@Override
	public void onError(SpeechError error) {
		//记录应用层错误码
		exit(error);
	}
	/**
	 * called from VAD check
	 */
	public void vadEndCall() {
		if (Status.recording == getStatus()) 
		{
			DebugLog.LogD( "ivw msc vadEndCall" );
			stopListening(false);
//			if (mListener != null)
//				mListener.onEndOfSpeech();
		}
	}
	/**
	 * 录音启动
	 */
	@Override
	public void onRecordStarted(boolean success) {}
	
	/**
	 * 录音机释放通知
	 */
	@Override
	public void onRecordReleased() {}

	@Override
	public String getSessionID() {
		return null;
	}

	/**
	 * 返回客户端ID
	 */
	@Override
	public String getClientID()
	{
		return mSession.getClientID();
	}
	
	/**
	 * 返回唤醒监听
	 */
	public WakeuperListener getListener()
	{
		return mListener;
	}
	
	public static int mergeSource(String params)
	{
		if(MSC.isLoaded() && !TextUtils.isEmpty(params))
		{
			HashParam param = new HashParam();
			param.putMultiParam(params);
			String src = param.getString(SpeechConstant.IVW_RES_PATH);
			String dest = param.getString(SpeechConstant.IVW_WORD_PATH);
			param.remove(SpeechConstant.IVW_RES_PATH);
			param.removeParam(SpeechConstant.IVW_WORD_PATH);
			if (src != null && dest != null)
			{
//				if( param.isEmpty())
//				{
//					return MSC.QIVWResMerge(src.getBytes(), dest.getBytes(), null);
//				}
//				else
				{
					DebugLog.LogD( "QIVWResMerge param: "+params );
					return MSC.QIVWResMerge(src.getBytes(), dest.getBytes(), params.toString().getBytes());
				}
			}
		}

		return -1;
			
	}



	private double getVolume(int r, byte[] mDataBuffer) {
		long v = 0;
		// 将 buffer 内容取出，进行平方和运算
		for(byte data:mDataBuffer) {
			v+=data*data;
		}
		// 平方和除以数据总长度，得到音量大小。
		double mean = v / (double) r;

		double volume = 10 * Math.log10(mean);
		if (volume>20) {
			volume = volume -20;
		}
		return volume;
	}
}