package com.iflytek.cloud.msc.ivw.aimic.impl;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.msc.util.log.DebugLog;

public class AudioRecorder {

	public static interface RecorderListener{
		public void onAudioData( byte[] audio, int dataLen );
	}
	/*******************************************************************/
	private static final String NM_CLS_ALSA = "com.iflytek.alsa.AlsaRecorder";
	private static final String NM_CLS_LISTENER = "com.iflytek.alsa.AlsaRecorder$PcmListener";
	private static final String NM_CLS_ALSA_JNI = "com.iflytek.alsa.jni.AlsaJni";
	
	private static final String NM_MTHD_CREATOR = "createInstance";
	private static final String NM_MTHD_START_RECORD = "startRecording";
	private static final String NM_MTHD_STOP_RECORD = "stopRecording";
	private static final String NM_MTHD_IS_RECORDING = "isRecording";
	
	private static final String NM_MTHD_GET_CARD_ID = "getCardDevId";
	private static final String NM_MTHD_GET_SAMPLE_RATE = "getSampleRate";
	private static final String NM_MTHD_GET_PERIOD_SIZE = "getPeriodSize";
	private static final String NM_MTHD_GET_LISTENER = "getListener";
	
	private static final String NM_MTHD_DESTROY = "destroy";
	
	private static final String NM_MTHD_ON_PCM = "onPcmData";
	private static final String NM_MTHD_SET_LOG = "showJniLog";
	/*******************************************************************/
	
	private static final Class<?>[] EMPTY_CLASSES = null;
	private static final Object[] EMPTY_OBJECTS = null;
	
	private Class<?> mClsAlsa = null;
	private Method mMthdStartRecord = null;
	private Method mMthdStopRecord = null;
	private Method mMthdIsRecording = null;
	
	private Method mMthdGetCardId = null;
	private Method mMthdGetSampleRate = null;
	private Method mMthdGetPeriodSize = null;
	private Method mMthdGetListener = null;
	
	private Method mMthdDestroy = null;
	
	private Class<?> mClsListener = null;
	private Method mMthdOnPcmData = null;
	
	private Class<?> mClsAlsaJni = null;
	private Method mMthdSetLog = null;
	
	private Object mAlsaRecorder = null;
	private final AlsaListenerInvocationHandler mAlsaListenerHandler = new AlsaListenerInvocationHandler();
	private Object mAlsaListener = null;
	/*******************************************************************/
	
	private static final Object sSync = new Object();
	private static AudioRecorder sInstance = null;
	
	private RecorderListener mListener = null;
	
	public static AudioRecorder createInstance( final int card, final int sampleRate, final int periodSize ){
		AudioRecorder recorder = null;
		
		synchronized( sSync ){
			if( null == sInstance ){
				try{
					sInstance = new AudioRecorder( card, sampleRate, periodSize );
				}catch( final Throwable e ){
					DebugLog.LogE( e );
				}//end of try-catch
			}
			
			recorder = sInstance;
		}//end of sync
		
		return recorder;
	}
	
	public static AudioRecorder getInstance(){
		AudioRecorder recorder = null;
		
		synchronized( sSync ){
			recorder = sInstance;
		}//end of sync
		
		return recorder;
	}
	
	public int startRecording( RecorderListener listener ){
		int ret = ErrorCode.SUCCESS;
		mListener = listener;
		
		try {
			ret = (Integer) this.mMthdStartRecord.invoke( mAlsaRecorder, mClsListener.cast(this.mAlsaListener) );
		} catch ( Throwable e) {
			DebugLog.LogE( e );
			ret = ErrorCode.ERROR_UNKNOWN;
		}//end of try-catch
		
		return ret;
	}
	
	public boolean isRecording(){
		boolean ret = false;
		
		try {
			ret = (Boolean) this.mMthdIsRecording.invoke( mAlsaRecorder, EMPTY_OBJECTS );
		} catch ( Throwable e) {
			DebugLog.LogE( e );
			ret = false;
		}//end of try-catch
		
		return ret;
	}
	
	public void stopRecording(){
		try {
			this.mMthdStopRecord.invoke( mAlsaRecorder, EMPTY_OBJECTS );
		} catch ( Throwable e) {
			DebugLog.LogE( e );
		}//end of try-catch
	}
	
	public int getCardDevId(){
		int ret = -1;

		try {
			ret = (Integer) this.mMthdGetCardId.invoke( mAlsaRecorder, EMPTY_OBJECTS );
		} catch ( Throwable e) {
			DebugLog.LogE( e );
		}//end of try-catch
		
		return ret;
	}

	public int getSampleRate() {
		int ret = -1;

		try {
			ret = (Integer) this.mMthdGetSampleRate.invoke(mAlsaRecorder, EMPTY_OBJECTS);
		} catch (Throwable e) {
			DebugLog.LogE(e);
		} // end of try-catch

		return ret;
	}

	public int getPeriodSize() {
		int ret = -1;

		try {
			ret = (Integer) this.mMthdGetPeriodSize.invoke(mAlsaRecorder, EMPTY_OBJECTS);
		} catch (Throwable e) {
			DebugLog.LogE(e);
		} // end of try-catch

		return ret;
	}

	public RecorderListener getListener() {
		RecorderListener listener = mListener;
		
		try {
			final Object alsaListener = this.mMthdGetListener.invoke(mAlsaRecorder, EMPTY_OBJECTS);
			if( !this.mAlsaListener.equals(alsaListener) ){
				listener = null;
				DebugLog.LogD( "Recorder getListener alsa listener unequal to current object: "+alsaListener );
			}
		} catch (Throwable e) {
			DebugLog.LogE(e);
			listener = null;
		} // end of try-catch
		
		return listener;
	}

	public void destroy() {
		try {
			this.mMthdDestroy.invoke(mAlsaRecorder, EMPTY_OBJECTS);
		} catch (Throwable e) {
			DebugLog.LogE(e);
		} // end of try-catch

		synchronized (sSync) {
			sInstance = null;
		} // end of sync
	}
	
	public void setShowLog( final boolean enabled ){
		try {
			this.mMthdSetLog.invoke(null, enabled);
		} catch (Throwable e) {
			DebugLog.LogE(e);
		} // end of try-catch
	}
	
	private AudioRecorder( final int card, final int sampleRate, final int periodSize )	throws NullPointerException
			, NoSuchMethodException
			, InvocationTargetException
			, IllegalAccessException
			, InstantiationException
			, ClassNotFoundException{
		mClsListener = Class.forName( NM_CLS_LISTENER );
		mMthdOnPcmData = mClsListener.getMethod( NM_MTHD_ON_PCM, byte[].class, int.class );
		mAlsaListener = Proxy.newProxyInstance( mClsListener.getClassLoader(), new Class[]{mClsListener}, this.mAlsaListenerHandler );
		
		mClsAlsa = Class.forName( NM_CLS_ALSA );
		final Method mthdCreatorAlsa = mClsAlsa.getMethod( NM_MTHD_CREATOR, int.class, int.class, int.class );
		mAlsaRecorder = mthdCreatorAlsa.invoke( null, card, sampleRate, periodSize );
		if( null == mAlsaRecorder ){
			DebugLog.LogE( "Recorder create alsa failed!" );
			throw new NullPointerException( "Recorder create alsa failed!" );
		}//end of if alsa is null
		
		mMthdStartRecord = mClsAlsa.getMethod( NM_MTHD_START_RECORD, mClsListener );
		mMthdStopRecord = mClsAlsa.getMethod( NM_MTHD_STOP_RECORD, EMPTY_CLASSES );
		mMthdDestroy = mClsAlsa.getMethod( NM_MTHD_DESTROY, EMPTY_CLASSES );
		mMthdGetCardId = mClsAlsa.getMethod( NM_MTHD_GET_CARD_ID, EMPTY_CLASSES );
		
		mMthdGetListener = mClsAlsa.getMethod( NM_MTHD_GET_LISTENER, EMPTY_CLASSES );
		mMthdGetPeriodSize = mClsAlsa.getMethod( NM_MTHD_GET_PERIOD_SIZE, EMPTY_CLASSES );
		mMthdGetSampleRate = mClsAlsa.getMethod( NM_MTHD_GET_SAMPLE_RATE, EMPTY_CLASSES );
		mMthdIsRecording = mClsAlsa.getMethod( NM_MTHD_IS_RECORDING, EMPTY_CLASSES );
		
		mClsAlsaJni = Class.forName( NM_CLS_ALSA_JNI );
		mMthdSetLog = mClsAlsaJni.getMethod( NM_MTHD_SET_LOG, boolean.class );
	}
	
	private class AlsaListenerInvocationHandler implements InvocationHandler {

		@Override
		public Object invoke( final Object proxy, final Method method, final Object[] args) throws Throwable {
			if( method.equals(mMthdOnPcmData) ){
				if( null != mListener ){
					mListener.onAudioData( (byte[])args[0], (Integer)args[1] );
				}
				
				return null;
			}
			
			return null;
		}//end of function invoke
		
	}//end of class AlsaListener
}
