package com.iflytek.cloud.msc.msp;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

import android.content.Context;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;

public class ComSession {

	// Com系列接口互斥
	public static Object synLock = new Object();
	
	public static void Login(Context context,String usr,String pwd,MscHandler looper) throws SpeechError, IOException {
		synchronized (synLock)
		{
			String param = ParamBuilder.composeInitParam(context,looper);
			byte[] btusr = null;
			byte[] btpwd = null;
			if(!TextUtils.isEmpty(usr))
				btusr = usr.getBytes(DataUtil.UTF8);
			else if(context != null)
			{
				String dusr = ParamBuilder.getDeviceId(context);
				if(TextUtils.isEmpty(dusr))
					btusr = null;
				else
				{
					btusr = dusr.getBytes(looper.getParamEncoding());
				}
			}
			if(!TextUtils.isEmpty(pwd))
				btpwd = pwd.getBytes(looper.getParamEncoding());
			
			int nRet = ErrorCode.SUCCESS;
			try {
			    nRet = MSC.QMSPLogin(btusr,btpwd,param.getBytes(looper.getParamEncoding()));
				// DebugLog.Log("[initCom]config:" + param);
				DebugLog.LogD("[MSPLogin]ret:" + nRet);
			}
			catch( UnsatisfiedLinkError e ){
				DebugLog.LogE( e );
				throw new SpeechError(ErrorCode.ERROR_UNSATISFIED_LINK );
			}

			if (0 != nRet)
				throw new SpeechError(nRet);
		}
	}

	public static boolean Logout() {
		// 等待其他所有会话退出
		if( !MscHandler.isEmpty() ){
			DebugLog.LogD( "MscHandler is not empty while logout." );
			return false;
		}
		
		synchronized (synLock) {
			int ret = MSC.QMSPLogOut();
			return (ret == 0);
		}
	}

	// *********************************************************

	/** member properties */
	private MSCSessionInfo mOutParams = new MSCSessionInfo();

	/**
	 * get audio data
	 * @param context 上下文信息
	 * @param name 文件名称
	 * @param data
	 *            storage for audio data
	 * @return
	 * @throws UnsupportedEncodingException 
	 */
	public synchronized byte[] uploadData(Context context,String name,byte[] data,MscHandler looper) throws SpeechError, UnsupportedEncodingException {
		synchronized (synLock) {
			byte[] ret = null;
			
			/** invoke native code */
			String sparam = ParamBuilder.composeMspSessionParam(context, looper);
			DebugLog.LogD("[MSPSession uploadData]enter time:" + System.currentTimeMillis());
			// DebugLog.Log("[MSPSession uploadData] param = " + sparam);
			ret = MSC.QMSPUploadData(name.getBytes(looper.getParamEncoding()), data,data.length, sparam.getBytes(DataUtil.UTF8),mOutParams);
			DebugLog.LogD("[MSPSession uploaddData]leavel:" + mOutParams.errorcode
					+ ",data len = " + (ret == null ? 0 : ret.length));

			int error = mOutParams.errorcode;
			/** raise exception */
			if (0 != error || ret == null) {
					throw new SpeechError(error);
			}
			return ret;
		}
	}
	
	/**
	 * get audio data
	 * 
	 * @param
	 *
	 * @return
	 * @throws UnsupportedEncodingException 
	 */
	public synchronized byte[] downLoadData(Context context,MscHandler looper) throws SpeechError, UnsupportedEncodingException {
		synchronized (synLock) {
			byte[] buffer = null;
			
			/** invoke native code */
			String sparam = ParamBuilder.composeMspSessionParam(context,looper);
			DebugLog.LogD("[MSPSession downloadData]enter time:" + System.currentTimeMillis());
			// DebugLog.Log("msp download param = " + sparam);
			PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
			buffer = MSC.QMSPDownloadData(sparam.getBytes(looper.getParamEncoding()), mOutParams);
			PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
			DebugLog.LogD("[MSPSession downloadData]leavel:" + mOutParams.errorcode
					+ ",data len = " + (buffer == null ? 0 : buffer.length));

			int error = mOutParams.errorcode;
			/** raise exception */
			if (0 != error || buffer == null) {
					throw new SpeechError(error);
			}
			return buffer;			
		}

	}

	/**
	 * search Text result
	 * 
	 * @param
	 * @return
	 * @throws UnsupportedEncodingException 
	 */
	public synchronized byte[] searchResult(Context context,MscHandler looper,String text) throws SpeechError, UnsupportedEncodingException {
		synchronized (synLock) {
			byte[] buffer = null;
			
			/** invoke native code */
			String sparam = ParamBuilder.composeMspSessionParam(context, looper);
			DebugLog.LogD("[MSPSession searchResult]enter time:" + System.currentTimeMillis());
			byte[] textbuffer = text.getBytes(DataUtil.UTF8);
			// DebugLog.Log("msp download param = " + sparam);
			PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
			buffer = MSC.QMSPSearch(sparam.getBytes(looper.getParamEncoding()),textbuffer, mOutParams);
			PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
			DebugLog.LogD("[QMSPSearch searchResult]leavel:" + mOutParams.errorcode
					+ ",data len = " + (buffer == null ? 0 : buffer.length));

			int error = mOutParams.errorcode;
			/** raise exception */
			if (0 != error || buffer == null) {
					throw new SpeechError(error);
			}
			return buffer;			
		}
	}
}
