package com.iflytek.cloud.msc.util;

import android.content.Context;
import android.content.SharedPreferences;
//import android.content.pm.PackageManager;
//import android.net.wifi.WifiInfo;
//import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Environment;
//import android.provider.Settings;
//import android.provider.Settings.Secure;
//import android.telephony.TelephonyManager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.lang.reflect.Method;
import java.security.MessageDigest;
import java.util.regex.Pattern;

import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;

public class UniqueIDUtil {
	// 校验码
	private static final String CHECK_CODE = "2";
	// Setting字段
	private static final String SETTING_KEY = "iflytek.deviceid.key";
	// 配置文件名：.MD5(iFlytekID)
	private static final String FILE_NAME = ".2F6E2C5B63F0F83B";
	// 配置Preferences名称
	private static final String PREFERENCES_NAME = "com.iflytek.id";
	// Preferences字段
	private static final String PREFERENCES_KEY = "pref.deviceid.key";
	// 是否使用WIFI MAC
	public static boolean sUseWifiMac = true;
	// MAC正则表达式模式
	//private static final Pattern PATTERN_MAC = Pattern.compile("^([0-9A-F]{2}:){5}([0-9A-F]{2})$");
	// ID正则表达式模式
	private static final Pattern PATTERN_ID = Pattern.compile("[0-3][0-9a-f]{32}");

	private static String sDeviceID = null;
	
	/**
	 * 获取唯一码
	 * 
	 * @param context
	 * @return
	 */
	public static synchronized String getUniqueID(Context context) {
		if (sDeviceID == null) {
			sDeviceID = achieveID(context);
		}
		return sDeviceID;
	}
	
	/**
	 * 清理已存信息
	 * 
	 * @param context
	 * @return
	 */
	public static void cleanRecord(Context context) {
		// 清理Setting
		putSettingString(context, SETTING_KEY, "");
		// 清理Preferences
		putPreferencesString(context, PREFERENCES_NAME, PREFERENCES_KEY, "");
		// 清理SDCard
		File file = new File(Environment.getExternalStorageDirectory() + "/msc", FILE_NAME);
		file.delete();
		// 清理静态变量
		sDeviceID = null;
	}

	/**
	 * 获取唯一码过程
	 * 
	 * @param context
	 * @return
	 */
	private static String achieveID(final Context context) {
		// 读取Setting
		String idSetting = getSettingString(context, SETTING_KEY);
		if (!checkInvalid(idSetting)) {
			return idSetting;
		}

		// 读取SDCard
		final String idSDCard = getSDCardString(context, FILE_NAME);
		if (!checkInvalid(idSDCard)) {
			new Thread(new Runnable() {
				public void run() {
					putSettingString(context, SETTING_KEY, idSDCard);
				}
			}).start();
			return idSDCard;
		}

		// 读取Preferences
		final String idPreferences = getPreferencesString(context, PREFERENCES_NAME, PREFERENCES_KEY, null);
		if (!checkInvalid(idPreferences)) {
			new Thread(new Runnable() {
				public void run() {
					putSettingString(context, SETTING_KEY, idPreferences);
					putSDCardString(context, FILE_NAME, idPreferences);
				}
			}).start();
			return idPreferences;
		}

		// 如果读取都无效，生成新ID
		final String id = createNewID(context);
		
		// 存储ID
		new Thread(new Runnable() {
			public void run() {
				putSettingString(context, SETTING_KEY, id);
				putSDCardString(context, FILE_NAME, id);
				putPreferencesString(context, PREFERENCES_NAME, PREFERENCES_KEY, id);
			}
		}).start();
		
		return id;
	}

	/**
	 * 读取保存在系统setting中的值
	 *
	 * @param context
	 * @param key
	 * @return
	 */
	private static String getSettingString(Context context, String key) {
		String value = "";
		try {
			value = android.provider.Settings.System.getString(context.getContentResolver(), key);
		} catch (Exception e) {
			DebugLog.LogS(e);
		}
		return value;
	}

	/**
	 * 将指定的键值对写入系统的setting中
	 *
	 * @param context
	 * @param key
	 * @param value
	 */
	private static void putSettingString(Context context, String key, String value) {
		try {
			android.provider.Settings.System.putString(context.getContentResolver(), key, value);
		} catch (Exception e) {
			DebugLog.LogS(e);
		}
	}

	/**
	 * 读取Preferences中的指定键值
	 * 
	 * @param context
	 * @param name
	 * @param key
	 * @param defValue
	 * @return
	 */
	private static String getPreferencesString(Context context, String name, String key, String defValue) {
		try {
			SharedPreferences preferences = context.getSharedPreferences(name, Context.MODE_PRIVATE);
			return preferences.getString(key, defValue);
		} catch (Throwable t) {
			DebugLog.LogS(t);
		}
		return defValue;
	}

	/**
	 * 向Preferences中写入指定键值对
	 * 
	 * @param context
	 * @param name
	 * @param key
	 * @param value
	 */
	private static void putPreferencesString(Context context, String name, String key, String value) {
		try {
			SharedPreferences preferences = context.getSharedPreferences(name, Context.MODE_PRIVATE);
			if (preferences != null) {
				SharedPreferences.Editor editor = preferences.edit();
				editor.putString(key, value);
				editor.commit();
			}
		} catch (Throwable t) {
			DebugLog.LogS(t);
		}
	}

	/**
	 * 从SD卡中读取文件内容
	 * 
	 * @param context
	 * @param fileName
	 * @return
	 */
	private static String getSDCardString(Context context, String fileName) {
		if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
			try {
				DebugLog.LogD("create file:"+fileName);
				File file = new File(Environment.getExternalStorageDirectory() + "/msc", fileName);	
				String str = readFile(file);
				return str;
			} catch (Throwable t) {
				DebugLog.LogS(t);
			}
		}
		return null;
	}

	/**
	 * 读取文件内容
	 * 
	 * @param paramFile
	 * @return
	 */
	private static String readFile(File paramFile) {
		try {
			if ((paramFile.exists()) && (paramFile.canRead())) {
				FileInputStream inputStream = new FileInputStream(paramFile);
				byte[] byteArray = new byte[33];
				int length = inputStream.read(byteArray);
				inputStream.close();
				return new String(byteArray, 0, length);
			}
		} catch (Throwable t) {
			DebugLog.LogS(t);
		}
		return null;
	}

	/**
	 * 向SD卡中文件写入内容
	 * 
	 * @param context
	 * @param fileName
	 * @param paramString
	 */
	private static void putSDCardString(Context context, String fileName, String paramString) {
		File file = new File(Environment.getExternalStorageDirectory() + "/msc", fileName);	
		writeFile(file, paramString);
	}

	/**
	 * 写文件函数
	 * 
	 * @param paramFile
	 * @param paramString
	 */
	private static void writeFile(File paramFile, String paramString) {
		try {
			FileOutputStream outputStream = new FileOutputStream(paramFile);
			outputStream.write(paramString.getBytes());
			outputStream.close();

			if (checkSDKVersion(9)) {
				Method method = paramFile.getClass().getMethod("setReadable",
						new Class[] { Boolean.TYPE, Boolean.TYPE });
				method.invoke(paramFile, new Object[] { Boolean.valueOf(true), Boolean.valueOf(false) });
			} else {
				Runtime.getRuntime().exec("chmod 444 " + paramFile.getAbsolutePath());
			}
		} catch (Throwable t) {
			DebugLog.LogS(t);
		}
	}

	/**
	 * 生成设备ID = 校验码 + 特征码 
	 * 特征码：设备字符串MD5码
	 * 
	 * @param context
	 * @return
	 */
	private static String createNewID(Context context) {
		String str = createDeviceString(context);
		str = CHECK_CODE + calMD5(str);
		return str;
	}

	/**
	 * 生成设备信息字符串
	 * 
	 * @param context
	 * @return
	 */
	private static String createDeviceString(Context context) {
		HashParam param = AppInfoUtil.getAppInfo(context);
		String imei = param.getString(AppInfoUtil.OS_IMEI);
		String androidId = param.getString(AppInfoUtil.OS_ANDROID_ID);
		String mac = param.getString(AppInfoUtil.getMac(context));

		if (null == imei && null == androidId && null == mac) {
			return System.currentTimeMillis() + "";
		}

		StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append(imei).append('-')
				.append(androidId);

		if(sUseWifiMac)
			stringBuilder.append('-').append(mac);
		String str = stringBuilder.toString();
		return str;
	}

//	/**
//	 * 检查APP权限是否开通
//	 * 
//	 * @param context
//	 * @param str
//	 * @return
//	 */
//	private static boolean checkPermission(Context context, String str) {
//		return context.checkCallingOrSelfPermission(str) == PackageManager.PERMISSION_GRANTED;
//	}

	/**
	 * 检查SDK版本是否满足大于INT
	 * 
	 * @param paramInt
	 * @return
	 */
	private static boolean checkSDKVersion(int paramInt) {
		return Build.VERSION.SDK_INT >= paramInt;
	}
	
//	/**
//	 * 获取IMEI
//	 * 
//	 * @param context
//	 * @return
//	 */
//	private static String getIMEI(Context context) {
//		try {
//			if (checkPermission(context, "android.permission.READ_PHONE_STATE")) {
//				TelephonyManager teleMgr = (TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE);
//				return teleMgr.getDeviceId();
//			}
//		} catch (Throwable t) {
//			DebugLog.LogE(t);
//		}
//		return null;
//	}
//
//	/**
//	 * 获取WIFI MAC
//	 * 
//	 * @param context
//	 * @return
//	 */
//	private static String getWifiMac(Context context) {
//		String str = null;
//		try {
//			if (checkPermission(context, "android.permission.ACCESS_WIFI_STATE")) {
//				WifiManager wifiMgr = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
//				if (wifiMgr.isWifiEnabled()) {
//					WifiInfo wifiInfo = wifiMgr.getConnectionInfo();
//					if (wifiInfo != null) {
//						str = wifiInfo.getMacAddress();
//						if (str != null) {
//							str = str.toUpperCase().trim();
//							if (str.equals("00:00:00:00:00:00") || !PATTERN_MAC.matcher(str).matches()) {
//								str = null;
//							}
//						}
//					}
//				}
//			}
//		} catch (Throwable t) {
//			DebugLog.LogE(t);
//		}
//		return str;
//	}
//
//	/**
//	 * 获取Android_id
//	 * 
//	 * @param context
//	 * @return
//	 */
//	private static String getAndroidID(Context context) {
//		try {
//			if (checkPermission(context, "android.permission.READ_PHONE_STATE")) {
//				return Settings.Secure.getString(context.getContentResolver(), Secure.ANDROID_ID);
//			}
//		} catch (Throwable t) {
//			DebugLog.LogE(t);
//		}
//		return null;
//	}

	/**
	 * 检查ID是否有效
	 * 
	 * @param str
	 * @return
	 */
	private static final boolean checkInvalid(String str) {
		return (str == null) || ("".equals(str.trim())) || !(PATTERN_ID.matcher(str).matches());
	}
	
	/**
	 * 计算MD5
	 * 
	 * @param str
	 * @return
	 */
	private static String calMD5(String str) {
		try {
			byte[] arrayOfByte = MessageDigest.getInstance("MD5").digest(str.getBytes("UTF-8"));
			return turnToHexString(arrayOfByte);
		} catch (Exception e) {
		}
		return null;
	}

	/**
	 * 转为16进制字符串
	 * 
	 * @param byteArray
	 * @return
	 */
	private static String turnToHexString(byte[] byteArray) {
		StringBuilder stringBuilder = new StringBuilder();
		int temp = 0;
		for (int i : byteArray) {
			temp = i & 0xFF;
			if (temp < 16) {
				stringBuilder.append('0');
			}
			stringBuilder.append(Integer.toHexString(temp));
		}
		return stringBuilder.toString();
	}
}
