package com.iflytek.cloud.speech.impl;


import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;

import com.iflytek.cloud.DataDownloader;
import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechListener;
import com.iflytek.cloud.VerifierListener;
import com.iflytek.cloud.VerifierResult;
import com.iflytek.cloud.msc.isv.ModelManager;
import com.iflytek.cloud.msc.isv.MscVerify;
import com.iflytek.cloud.msc.module.SpeechModule;
import com.iflytek.cloud.msc.util.FileUtil;
import com.iflytek.cloud.msc.util.FuncAdapter;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.MscKeys;

import java.security.SecureRandom;

public class SpeakerVerifierImpl extends SpeechModule{
	
	// 是否会话过程中暂停后台音乐播放
	private boolean mRequestFocus = false;
	
	private static final String allChar = "023456789";
	/**
	 * 初始化声纹对象。<br>
	 * @param context 上下文
	 */
	public SpeakerVerifierImpl(Context context) {
		super(context);
	}
	
	
	public int startListening(VerifierListener listener)
	{
		int ret = ErrorCode.SUCCESS;
		
		synchronized (mSynObj) {
			try{
				mRequestFocus = mSessionParams.getBoolean(SpeechConstant.KEY_REQUEST_FOCUS, true);
				
				if(null != mscer && mscer.isRunning())
	    		{
	    			// 判断是否通知外部识别被异常打断
	            	mscer.cancel(mSessionParams.getBoolean(SpeechConstant.ISV_INTERRUPT_ERROR, false));
	    		}
				
				mscer = new MscVerify(mContext,mSessionParams,getHandlerThread( "verify" ));
				//识别会话开始，暂停后台音乐播放.
				FuncAdapter.Lock(mContext, mRequestFocus, null);
				((MscVerify)mscer).startListening(new InternalListener(listener));
			}catch(SpeechError e){
				ret = e.getErrorCode();
				DebugLog.LogE( e );
			}catch( Throwable e ){
				ret = ErrorCode.ERROR_UNKNOWN;
				DebugLog.LogE( e );
			}//end of try-catch
		}//end of synchronized
		
		return ret;
	}
	
	/**
	 * Stops listening for speech.
	 */
	public void stopListening() {
		synchronized (mSynObj) {
			if (mscer != null) {
				((MscVerify)mscer).stopListening();	
			}
		}
	}
	/**
	 * 下载密码文本
	 * @param listener 监听器
	 */
	public void getPasswordList(final SpeechListener listener)
	{
		DataDownloader downer = new DataDownloader(mContext);
		downer.setParameter(mSessionParams);
		downer.downloadData(listener);
	}
	
	
   
	/**
     * 返回一个定长的随机字符串(只包含数字，去除1,2和5不邻接)
     * 
     * @param length
     *            随机字符串长度
     * @return 随机字符串
     */
	public String generatePassword(int length) {
        StringBuffer sb = new StringBuffer();
        SecureRandom random = new SecureRandom();
        //生成第一个数字
        String selected=allChar.charAt(random.nextInt(allChar.length()))+"";
        sb.append(selected);
        
        //生成余下的数字
        for (int i = 0; i < length-1; i++) {
        	Boolean isOkBoolean=false;
            while(!isOkBoolean) {
            	selected=allChar.charAt(random.nextInt(allChar.length()))+"";
            	if(sb.indexOf(selected)>=0)//该数字不与之前的重复
            	{
            		isOkBoolean=false;
            		continue;
            	}
            	if((Integer.parseInt(sb.charAt(sb.length()-1)+"")*Integer.parseInt(selected))==10)//该数字与前一个数字乘积!=10
            	{
            		isOkBoolean=false;continue;
            	}
            	isOkBoolean=true;
			}
        	sb.append(selected);
        }
        return sb.toString();
    }
	
	private final class InternalListener implements VerifierListener{
		private VerifierListener mOutListener = null;
		private static final int MSG_ERROR = 0;
		private static final int MSG_AUDIO_BUFFER = 1;
		private static final int MSG_SPEECH_BEGIN = 2;
		private static final int MSG_SPEECH_END = 3;
		private static final int MSG_RESULT = 4;
		private static final int MSG_EVENT = 5;
		
		public InternalListener(VerifierListener outlistener) {
			mOutListener = outlistener;
		}

		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (mOutListener == null)
					return;
				switch (msg.what) {
				case MSG_ERROR:
					mOutListener.onError((SpeechError) msg.obj);
					break;
				case MSG_RESULT:
					mOutListener.onResult((VerifierResult) msg.obj);
					break;
				case MSG_AUDIO_BUFFER:
					mOutListener.onVolumeChanged(msg.arg1,(byte[])msg.obj);
					break;
				case MSG_SPEECH_BEGIN:
					mOutListener.onBeginOfSpeech();
					break;
				case MSG_SPEECH_END:
					mOutListener.onEndOfSpeech();
					break;
				case MSG_EVENT:
					Message msgBody = (Message) msg.obj;
					mOutListener.onEvent(msgBody.what, msgBody.arg1, msgBody.arg2, (Bundle) msgBody.obj);
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			}
		};

		@Override
		public void onVolumeChanged(int volume, byte[] data) {
			Message msg = mUiHandler.obtainMessage(MSG_AUDIO_BUFFER, volume, 0,data);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onEndOfSpeech() {
			Message msg = mUiHandler.obtainMessage(MSG_SPEECH_END, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onBeginOfSpeech() {
			Message msg = mUiHandler
					.obtainMessage(MSG_SPEECH_BEGIN, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onResult(VerifierResult result) {
			sessionEnd();
			Message msg = mUiHandler.obtainMessage(MSG_RESULT,result);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onError(SpeechError error) {
			sessionEnd();
			Message msg = mUiHandler.obtainMessage(MSG_ERROR, error);
			mUiHandler.sendMessage(msg);
		}
		/**
		 * 会话结束
		 * 判断是否需要保存音频或继续音乐播放
		 * 该接口不对外开放.
		 */
		protected void sessionEnd()
		{
			String recordFile = mscer.getParam().getString(SpeechConstant.ISV_AUDIO_PATH);
			if(!TextUtils.isEmpty(recordFile)){
				if(FileUtil.saveFile(((MscVerify)mscer).getBuffer(), recordFile)){
					String format = mscer.getParam().getString(SpeechConstant.AUDIO_FORMAT, null);
	            	int sample = mscer.getParam().getInt(SpeechConstant.SAMPLE_RATE, mscer.mSampleRate);
	                FileUtil.formatPcm(format, recordFile, sample);
				}
			}
				
			//会话结束，后台音乐继续播放.
			FuncAdapter.UnLock(mContext, mRequestFocus, null);
		}

		@Override
		public void onEvent(int eventType, int arg1, int arg2, Bundle obj) {
			Message msgBody = Message.obtain();
			msgBody.what = eventType;
			msgBody.arg1 = arg1;
			msgBody.arg2 = arg2;
			msgBody.obj = obj;
			
			Message.obtain(mUiHandler, MSG_EVENT, msgBody).sendToTarget();
		}
	}

	public int writeAudio(byte[] buffer, int offset, int length) {
		synchronized (mSynObj) {
			if (mscer == null) {	
				DebugLog.LogD("writeAudio error, no active session.");
				return ErrorCode.ERROR_ENGINE_CALL_FAIL;
			}
			
			if(buffer == null || buffer.length <=0) {
				DebugLog.LogD("writeAudio error,buffer is null.");
				return ErrorCode.MSP_ERROR_INVALID_DATA;
			}
			
			if(buffer.length < (length + offset)) {
				DebugLog.LogD("writeAudio error,buffer length < length.");
				return ErrorCode.MSP_ERROR_INVALID_DATA;
			}
			if(((MscVerify)mscer).getAudioSource() != MscKeys.VALUE_AUDIO_EXTERNAL)
				return ErrorCode.MSP_ERROR_INVALID_PARA;
			
			((MscVerify)mscer).onRecordBuffer(buffer, offset, length);
			return ErrorCode.SUCCESS;	
		}
	}

	public boolean isListening() {
		return isRunning();
	}

	public int sendRequest(String cmd, String auth_id, SpeechListener listener) {
		int ret = ErrorCode.SUCCESS;
		
		synchronized (mSynObj) {
			try{
				mSessionParams.putParam(SpeechConstant.ISV_CMD, cmd);
				mSessionParams.putParam(SpeechConstant.AUTH_ID, auth_id);
				
				ModelManager manager = new ModelManager(mContext, getHandlerThread("manager"));
				manager.sendRequest(mSessionParams, new ModelManager.InnerSpeechListener(listener));
			}catch(SpeechError e){
				ret = e.getErrorCode();
				DebugLog.LogE( e );
			}catch( Throwable e ){
				ret = ErrorCode.ERROR_UNKNOWN;
				DebugLog.LogE( e );
			}//end of try-catch
		}//end of synchronized
		
		return ret;
	}
}
