/*
 * 文件名：SpeechRecognizerImpl.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：识别对象具体实现.
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud.speech.impl;

import java.util.concurrent.ConcurrentLinkedQueue;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.GrammarListener;
import com.iflytek.cloud.LexiconListener;
import com.iflytek.cloud.RecognizerListener;
import com.iflytek.cloud.RecognizerResult;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.isr.IsrSession;
import com.iflytek.cloud.msc.isr.MscRecognizer;
import com.iflytek.cloud.msc.isr.MscRecognizerMeta;
import com.iflytek.cloud.msc.module.SessPerflog;
import com.iflytek.cloud.msc.module.SpeechModule;
import com.iflytek.cloud.msc.util.FileUtil;
import com.iflytek.cloud.msc.util.FuncAdapter;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.util.AudioDetector;

public class SpeechRecognizerImpl extends SpeechModule {

    // 是否会话过程中暂停后台音乐播放
    private boolean mRequestFocus = false;

    public SpeechRecognizerImpl(final Context context) {
        super(context);
    }

    /**
     * 性能结构化日志插入
     * 
     * @param key
     */
    public void pushSessionInfo(final String key) {
        synchronized (mSynObj) {
            if (null != mscer) {
                ((MscRecognizer) mscer).getPerfLog().pushSessionInfo(key);
            }
        }
    }

    /**
     * 
     * @param listener
     *            回调监听
     * @return void
     */
    public int startListening(final RecognizerListener listener) {
    	int ret = ErrorCode.SUCCESS;
    	
        synchronized (mSynObj) {
        	try{
                mRequestFocus = mSessionParams.getBoolean(
                        SpeechConstant.KEY_REQUEST_FOCUS, true);
                if(null!=mscer && mscer.isRunning())
        		{
        			// 判断是否通知外部识别被异常打断
                	mscer.cancel(mSessionParams.getBoolean(SpeechConstant.ASR_INTERRUPT_ERROR, false));
        		}
                
                if( isBosDispose() ){
                	mscer = new MscRecognizerMeta(mContext, mSessionParams,
                            getHandlerThread("iat"));
                }else{
                	mscer = new MscRecognizer(mContext, mSessionParams,
                            getHandlerThread("iat"));
                }
                
                
                // 识别会话开始，暂停后台音乐播放.
                FuncAdapter.Lock(mContext, mRequestFocus, null);
                ((MscRecognizer) mscer).startListening(new InternalListener(
                        listener));
			}catch(SpeechError e){
				ret = e.getErrorCode();
				DebugLog.LogE( e );
			}catch( Throwable e ){
				ret = ErrorCode.ERROR_UNKNOWN;
				DebugLog.LogE( e );
			}//end of try-catch
        }//end of synchronized
        
        return ret;
    }

    /**
     * 写入录音数据
     * 
     * @param buffer
     * @return
     */
    public int writeAudio(final byte[] buffer, final int offset,
            final int length) {
        synchronized (mSynObj) {
            if (mscer == null) {
                DebugLog.LogD("writeAudio error, no active session.");
                return ErrorCode.ERROR_ENGINE_CALL_FAIL;
            }

            if (buffer == null || buffer.length <= 0) {
                DebugLog.LogD("writeAudio error,buffer is null.");
                return ErrorCode.MSP_ERROR_INVALID_DATA;
            }

            if (buffer.length < (length + offset)) {
                DebugLog.LogD("writeAudio error,buffer length < length.");
                return ErrorCode.MSP_ERROR_INVALID_DATA;
            }

            if (((MscRecognizer) mscer).getAudioSource() != MscKeys.VALUE_AUDIO_EXTERNAL) {
                return ErrorCode.MSP_ERROR_INVALID_PARA;
            }

            return ((MscRecognizer) mscer).writeAudio(buffer, offset, length);
        }
    }

    /**
     * Stops listening for speech.
     */
    public void stopListening() {
        synchronized (mSynObj) {
            if (mscer != null) {
                ((MscRecognizer) mscer).stopListening(true);
            }
        }
    }

    /**
     * 取消消息，保存底层事件
     */
    @Override
    public void cancel(final boolean notifyError) {
    	synchronized( mSynObj ){
    		sessionEnd();
            super.cancel(notifyError);
    	}//end of sync
    }

    /**
     * return audio record buffer
     */
    public ConcurrentLinkedQueue<byte[]> getLastBuffer() {
        synchronized (mSynObj) {
            if (mscer != null) {
                return ((MscRecognizer) mscer).getBuffer();
            }
            return null;
        }
    }
    
    /**
     * 会话结束 判断是否需要保存音频或继续音乐播放 该接口不对外开放.
     */
    protected void sessionEnd() {
    	if( null != mscer ){
    		String recordFile = mscer.getParam().getString(
                    SpeechConstant.ASR_AUDIO_PATH);
            // 保存到本地文件中,只有设置文件路径生效
            if (!TextUtils.isEmpty(recordFile)) {
            	if(FileUtil.saveFile(((MscRecognizer) mscer).getBuffer(),recordFile)){
            		String format = mscer.getParam().getString(SpeechConstant.AUDIO_FORMAT, null);
            		int sample = mscer.getParam().getInt(SpeechConstant.SAMPLE_RATE, mscer.mSampleRate);
            		FileUtil.formatPcm(format, recordFile, sample);
            	}
            }
    	}//end of if mscer not null
        
        // 识别会话结束，后台音乐继续播放.
        FuncAdapter.UnLock(mContext, mRequestFocus, null);
    }

    private final class InterUpdateLexicon implements LexiconListener {

        private LexiconListener  mOutListener  = null;
        private static final int GRAMMAR_ERROR = 0;
        private static final int GRAMMARID     = 1;

        public InterUpdateLexicon(final LexiconListener outlistener) {
            mOutListener = outlistener;
        }

        private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
                                       @Override
                                       public void handleMessage(
                                               final Message msg) {
                                           if (mOutListener == null) {
                                               return;
                                           }
                                           switch (msg.what) {
                                           case GRAMMAR_ERROR:
                                               mOutListener.onLexiconUpdated(
                                                       null,
                                                       (SpeechError) msg.obj);
                                               break;
                                           case GRAMMARID:
                                               mOutListener.onLexiconUpdated(
                                                       (String) msg.obj, null);
                                               break;

                                           default:
                                               break;
                                           }
                                           super.handleMessage(msg);
                                       }
                                   };

        @Override
        public void onLexiconUpdated(final String lexiconId,
                final SpeechError error) {
            if (error != null) {
                Message msg = mUiHandler.obtainMessage(GRAMMAR_ERROR, error);
                mUiHandler.sendMessage(msg);
            } else {
                Message msg = mUiHandler.obtainMessage(GRAMMARID, lexiconId);
                mUiHandler.sendMessage(msg);
            }

        }

    }

    private final class InterGrammarListener implements GrammarListener {

        private GrammarListener  mOutListener  = null;
        private static final int GRAMMAR_ERROR = 0;
        private static final int GRAMMARID     = 1;

        public InterGrammarListener(final GrammarListener outlistener) {
            mOutListener = outlistener;
        }

        private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
                                       @Override
                                       public void handleMessage(
                                               final Message msg) {
                                           if (mOutListener == null) {
                                               return;
                                           }
                                           switch (msg.what) {
                                           case GRAMMAR_ERROR:
                                               mOutListener.onBuildFinish(null,
                                                       (SpeechError) msg.obj);
                                               break;
                                           case GRAMMARID:
                                               mOutListener.onBuildFinish(
                                                       (String) msg.obj, null);
                                               break;

                                           default:
                                               break;
                                           }
                                           super.handleMessage(msg);
                                       }
                                   };

        @Override
        public void onBuildFinish(final String grammarId,
                final SpeechError error) {

            if (error != null) {
                Message msg = mUiHandler.obtainMessage(GRAMMAR_ERROR, error);
                mUiHandler.sendMessage(msg);
            } else {
                Message msg = mUiHandler.obtainMessage(GRAMMARID, grammarId);
                mUiHandler.sendMessage(msg);
            }

        }

    }

    private final class InternalListener implements RecognizerListener {
        private RecognizerListener mOutListener     = null;
        private static final int   MSG_ERROR        = 0;
        private static final int   MSG_AUDIO_BUFFER = 1;
        private static final int   MSG_SPEECH_BEGIN = 2;
        private static final int   MSG_SPEECH_END   = 3;
        private static final int   MSG_RESULT       = 4;
        // EVENT自定义消息
        private static final int   MSG_EVENT        = 6;
        private boolean            isFirstResult    = false;

        public InternalListener(final RecognizerListener outlistener) {
            mOutListener = outlistener;
        }

        private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
        	@Override
        	public void handleMessage(
        			final Message msg) {
        		if (mOutListener == null) {
        			return;
        		}
        		switch (msg.what) {
        		case MSG_ERROR:
        			mOutListener
        			.onError((SpeechError) msg.obj);
        			break;
        		case MSG_AUDIO_BUFFER:
        			mOutListener
        			.onVolumeChanged(msg.arg1 ,(byte[])msg.obj);
        			break;
        		case MSG_SPEECH_BEGIN:
        			mOutListener.onBeginOfSpeech();
        			break;
        		case MSG_SPEECH_END:
        			mOutListener.onEndOfSpeech();
        			break;
        		case MSG_RESULT: {
        			mOutListener
        			.onResult(
        					(RecognizerResult) msg.obj,
        					msg.arg1 == 1 ? true
        							: false);
        			if (false == isFirstResult) {
        				pushSessionInfo(SessPerflog.KEY_UI_FRS);
        				isFirstResult = true;
        			}
        			if (1 == msg.arg1) {
        				pushSessionInfo(SessPerflog.KEY_UI_LRS);
        			}
        		}
        		break;
        		case MSG_EVENT:
        			Message mbody = (Message) msg.obj;
        			// 自定义消息数据
        			mOutListener.onEvent(mbody.what,
        					mbody.arg1, mbody.arg2,(Bundle)mbody.obj);
        			break;
        		default:
        			break;
        		}
        		super.handleMessage(msg);
        	}
        };

        @Override
		public void onVolumeChanged(int volume, byte[] data) {
        	 Message msg = mUiHandler.obtainMessage(MSG_AUDIO_BUFFER, volume, 0,
        			 data);
             mUiHandler.sendMessage(msg);
			
		}
        @Override
        public void onEndOfSpeech() {
            Message msg = mUiHandler.obtainMessage(MSG_SPEECH_END, 0, 0, null);
            mUiHandler.sendMessage(msg);
        }

        @Override
        public void onResult(final RecognizerResult result, final boolean isLast) {
            // 当返回最后一个结果，判断是否需要保存音频或继续音乐播放
            if (isLast) {
                sessionEnd();
            }
            Message msg = mUiHandler.obtainMessage(MSG_RESULT,
                    isLast == true ? 1 : 0, 0, result);
            mUiHandler.sendMessage(msg);
        }

        @Override
        public void onBeginOfSpeech() {
            DebugLog.LogD("onBeginOfSpeech");
            Message msg = mUiHandler
                    .obtainMessage(MSG_SPEECH_BEGIN, 0, 0, null);
            mUiHandler.sendMessage(msg);
        }

        @Override
        public void onError(final SpeechError error) {
            sessionEnd();
            Message msg = mUiHandler.obtainMessage(MSG_ERROR, error);
            mUiHandler.sendMessage(msg);
        }

        @Override
        public void onEvent(final int eventType, final int arg1,
                final int arg2, final Bundle obj) {

            Message msgbody = new Message();
            msgbody.what = eventType;
            msgbody.arg1 = arg1;
            msgbody.arg2 = arg2;
            msgbody.obj = obj;

            Message tmsg = mUiHandler.obtainMessage(MSG_EVENT, 0, 0, msgbody);
            mUiHandler.sendMessage(tmsg);
        }
    }

    public boolean isListening() {
        return isRunning();
    }

    public int buildGrammar(final String grammarType,
            final String grammarContent, final GrammarListener grammarListener) {

        // 无效的文本输入
        if (TextUtils.isEmpty(grammarContent)) {
            return ErrorCode.ERROR_EMPTY_UTTERANCE;
        }

        // 无效的参数
        if (TextUtils.isEmpty(grammarType)) {
            return ErrorCode.ERROR_INVALID_PARAM;
        }

        // 无效的参数
        if (grammarListener == null) {
            return ErrorCode.ERROR_INVALID_PARAM;
        }

        IsrSession isrSession = new IsrSession();
        isrSession.buildGrammar(grammarType, grammarContent,
                new InterGrammarListener(grammarListener), mSessionParams);
        return ErrorCode.SUCCESS;
    }

    public int updateLexicon(final String lexiconName,
            final String lexiconContent, final LexiconListener lexiconListener) {
        // 无效的文本输入
        if (TextUtils.isEmpty(lexiconContent)) {
            return ErrorCode.ERROR_EMPTY_UTTERANCE;
        }

        // 无效的参数
        if (TextUtils.isEmpty(lexiconName)) {
            return ErrorCode.ERROR_INVALID_PARAM;
        }

        // 无效的参数
        if (lexiconListener == null) {
            return ErrorCode.ERROR_INVALID_PARAM;
        }

        IsrSession isrSession = new IsrSession();
        mSessionParams.putParam(SpeechConstant.SUBJECT, "uup",false);
        String lexiconType = getParameter(SpeechConstant.LEXICON_TYPE);
        if (TextUtils.isEmpty(lexiconType)) {
            lexiconType = lexiconName;
        }
        mSessionParams.putParam(SpeechConstant.DATA_TYPE, lexiconType,false );
        isrSession.upDateLexicon(lexiconName, lexiconContent,
                new InterUpdateLexicon(lexiconListener), mSessionParams);
        return ErrorCode.SUCCESS;
    }

    protected boolean isBosDispose(){
    	boolean ret = false;
    	
    	final String disposeValue = mSessionParams.getString( MscKeys.BOS_DISPOSE );
    	if( TextUtils.isEmpty(disposeValue) ){
    		ret = AudioDetector.TYPE_META.equalsIgnoreCase( mSessionParams.getString(AudioDetector.VAD_ENGINE) );
    	}else{
    		ret = mSessionParams.getBoolean( MscKeys.BOS_DISPOSE, ret );
    	}
    	
    	return ret;
    }
}
