package com.iflytek.cloud.util.code.impl;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;

import android.content.Context;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.util.AudioCoder;
import com.iflytek.cloud.util.code.impl.Codec.AudioData;

public class AudioCoderImpl extends AudioCoder {
	
	public static class Library{
		public static final String NAME_SPEEX = "speex";
		public static final String NAME_AMR = "amr";
		public static final String NAME_AMR_FX = "amr_fx";
		public static final String NAME_AMR_WB = "amr_wb";
		public static final String NAME_AMR_WB_FX = "amr_wb_fx";
	}
	
	private final static Map<String, Class<?>> sLibs = new HashMap<String, Class<?>>();
	
	static{
		//TODO：新增的库时，只需要把从Codec继承的类，加入到sLibs中
		sLibs.put( Library.NAME_SPEEX, SpeexCodec.class );
	}
	
	public static final String DEF_LIBRARY = Library.NAME_SPEEX;
	
	private Codec mCodec = null;
	private HashParam mHashParam = new HashParam( );
	private short mLevel = Codec.DEF_LEVEL;
	
	private boolean mIsEncodeInit = false;
	private Codec.InstanceInfo mEncodeInfo = new Codec.InstanceInfo();
	private AudioData mEncodeData = new AudioData( MAX_BUF_LEN*128);
	private byte[] mEncodeOrigin = new byte[MAX_BUF_LEN];
	private CoderResult mEncodeResult = new CoderResult();
	
	private boolean mIsDecodeInit = false;
	private Codec.InstanceInfo mDecodeInfo = new Codec.InstanceInfo();
	private AudioData mDecodeData = new AudioData( MAX_BUF_LEN*128 );;
	private byte[] mDecodeOrigin = new byte[MAX_BUF_LEN];
	private CoderResult mDecodeResult = new CoderResult();
	
	
	public static AudioCoder createCoder( Context context, String param ){
		DebugLog.LogD( "Create coder enter, context: "+context+", param: "+param );
		AudioCoderImpl instance = null;
		synchronized( sSyncObj ){
			instance = new AudioCoderImpl( context, param );
			if( null == instance.mCodec ){
				instance = null;
			}
		}//end of synchronized
		
		DebugLog.LogD( "Create coder leave." );
		return instance;
	}

	public AudioCoderImpl(Context context, String param) {
		super(context, param);
		
		mHashParam.initWithParam( param );
		
		mCodec = createCodec( mHashParam.getString(AudioCoder.LIBRARY, DEF_LIBRARY)
				, mHashParam.getString(SpeechConstant.LIB_NAME) );
	}
	
	@Override
	public CoderResult encode(byte[] buffer, int offset, int length,
			boolean isLast) {
		DebugLog.LogD( "Encode enter, buffer: "+buffer+", offset: "+offset+", length: "+length+", iaLast: "+isLast );
		
		synchronized( sSyncObj ){
			if( !mIsEncodeInit && null!=mCodec ){
				final short mode = (short)mHashParam.getInt( AudioCoder.MODE, Codec.DEF_MODE );
				mEncodeResult.error = mCodec.encodeInit( mEncodeInfo, mode );
				DebugLog.LogD( "encode init ret:"+mDecodeInfo.errorCode );
				mIsEncodeInit = ErrorCode.SUCCESS==mEncodeInfo.errorCode;
			}
			
			if( mIsEncodeInit ){
				code( buffer, offset, length, isLast, true );
			}
			
			if( isLast && null!=mCodec && mIsEncodeInit ){
				mCodec.encodeFini( mEncodeInfo );
				DebugLog.LogD( "encode fini ret:"+mEncodeInfo.errorCode );
				mIsEncodeInit = false;
			}
			
		}//end of synchronized
		
		DebugLog.LogD( "Encode leave" );
		return mEncodeResult;
	}

	@Override
	public CoderResult decode(byte[] buffer, int offset, int length,
			boolean isLast) {
		DebugLog.LogD( "Decode enter, buffer: "+buffer+", offset: "+offset+", length: "+length+", iaLast: "+isLast );
		
		synchronized( sSyncObj ){
			if( !mIsDecodeInit && null!=mCodec ){
				final short mode = (short)mHashParam.getInt( AudioCoder.MODE, Codec.DEF_MODE );
				mDecodeResult.error = mCodec.decodeInit( mDecodeInfo, mode );
				DebugLog.LogD( "decode init ret:"+mDecodeInfo.errorCode );
				mIsDecodeInit = ErrorCode.SUCCESS==mDecodeInfo.errorCode;
			}
			
			if( mIsDecodeInit ){
				code( buffer, offset, length, isLast, false );
			}
			
			if( isLast && null!=mCodec && mIsDecodeInit ){
				mCodec.decodeFini( mDecodeInfo );
				DebugLog.LogD( "decode fini ret:"+mDecodeInfo.errorCode );
				mIsDecodeInit = false;
			}
		}//end of synchronized
		
		DebugLog.LogD( "Decode leave" );
		return mDecodeResult;
	}

	@Override
	public void setParameter(String key, String value) {
		DebugLog.LogD( "Set coder parameter enter, key: "+key+", value: "+value );
		
		synchronized( sSyncObj ){
			if( !TextUtils.isEmpty(key) ){
				if( TextUtils.isEmpty(value) ){
					mHashParam.remove( key );
				}else{
					mHashParam.putParam( key, value );
				}//end of if-else value is empty
				
				mLevel = (short)mHashParam.getInt( LEVEL, mLevel );
			}//end of if key is not empty
		}//end of synchronized
		
		DebugLog.LogD( "Set coder parameter leave" );
	}

	@Override
	public boolean destroy() {
		DebugLog.LogD( "Destroy coder enter" );
		
		synchronized( sSyncObj ){
			if( null != mCodec ){
				if( mIsEncodeInit ){
					mCodec.encodeFini( mEncodeInfo );
				}
				
				if( mIsDecodeInit ){
						mCodec.decodeFini( mDecodeInfo );	
				}
				
				mCodec = null;
			}
			
			sInstance = null;
		}//end of synchronized
		
		DebugLog.LogD( "Destroy coder leave" );
		return true;
	}
	
	private static Codec createCodec( String name, String loadName ){
		Codec instance= null;
		
		if( loadLibrary(loadName) ){
			instance = createCodec( sLibs.get(name) );
		}//end of is loadLibrary succeed
		
		return instance;
	}//end of function createInstance
	
	private static Codec createCodec( final Class<?> cls ){
		Codec codec = null;
		
		try {
			if( null != cls ){
				Constructor<?> constructor = cls.getConstructor();
				Object obj = constructor.newInstance();
				if( null != obj 
						&& obj instanceof Codec ){
					codec = (Codec)obj;
				}
			}else{
				DebugLog.LogE( "create Codec cls is null !" );
			}//end of if-else cls is not null
		} catch (Exception e) {
			DebugLog.LogE( e.getLocalizedMessage() );
		}//end of try-catch
		
		return codec;
	}

	private static  boolean loadLibrary( String loadName ){
		boolean ret = true;
		if( !TextUtils.isEmpty(loadName) ){
			try{
				System.loadLibrary( loadName );
			}catch(Exception e){
				ret = false;
				DebugLog.LogE( "Load library failed." );
				e.printStackTrace();
			}
		}//end of if loadName is empty
		
		return ret;
	}//end of function loadLibrary

	private static final void resetResult( final CoderResult result ){
		result.buffer = null;
		result.error = 0;
		result.length = 0;
		result.offset = 0;
	}

	private void code( byte[] buffer, int offset, int length,
			boolean isLast, boolean isEncode ){
		AudioData audioData = null;
		CoderResult result = null;
		byte[] origin = null;
		Codec.InstanceInfo instanceInfo = null;
		if( isEncode ){
			audioData = mEncodeData;
			result = mEncodeResult;
			origin = mEncodeOrigin;
			instanceInfo = mEncodeInfo;
		}else{
			audioData = mDecodeData;
			result = mDecodeResult;
			origin = mDecodeOrigin;
			instanceInfo = mDecodeInfo;
		}//end of if-else is encode
		
		audioData.reset();
		resetResult( result );
		
		if( null == mCodec ){
			DebugLog.LogE( "code error: codec is null!" );
			result.error = ErrorCode.ERROR_ENGINE_INIT_FAIL;
		}else if( null==buffer 
				|| 0>=length 
				|| MAX_BUF_LEN<length
				|| 0>offset
				|| buffer.length-offset<length){
			 if( isLast ){
				 result.error = ErrorCode.SUCCESS;
			 }else{
				 result.error = ErrorCode.ERROR_INVALID_PARAM;
			 }//end of if-else is last
		}else{
//			DebugLog.LogD( "audioData buffer="+audioData.dataBuf+", len="+audioData.bufLen );
			System.arraycopy( buffer, offset, origin, 0, length );
			if( isEncode ){
				result.error = mCodec.encode( instanceInfo, origin, length, audioData, mLevel );
			}else{
				result.error = mCodec.decode( instanceInfo, origin, length, audioData );
			}//end of if-else is encode
			DebugLog.LogD( "code ret: "+result.error );
			
			if( ErrorCode.SUCCESS == result.error ){
				result.buffer = audioData.dataBuf;
				result.length = audioData.dataLen;
			}//end of if is succeed
		}//end of if-else is parameter valid
		
	}//end of function code
}
